package app;

import java.awt.event.*;
import javax.swing.*;
import java.util.ArrayList;
import java.lang.reflect.*;

/**
 * Verwaltet das Men einer Applikation.<br>
 * <i>(C)opyright 2002 L.H.Hahn</i>
 * @author Dr. Lars H. Hahn
 * @version 3.0
 * @see Applikation
 */

public class MenuHandler
  extends JMenuBar
  implements ActionListener {

  private final boolean DEBUG = false;

  private Applikation applikation;
  private ArrayList menus = new ArrayList();
  private ArrayList menuItems = new ArrayList();

  /**
   * @param applikation Applikation, die mit dem Men gesteuert werden soll.
   */
  public MenuHandler(Applikation applikation) {
    this.applikation = applikation;
    if(DEBUG)
      System.out.println("MenuHandler initialisiert.");
  }

  /**
   * Setzt den Zustand einer Check-Box im Men.
   * @param actionCommand Schlssel zur Auswahl der Check-Box
   * @param state neuer Zustand der Check-Box
   */
  public void setState(String actionCommand, boolean state) {
    for(int i=0; i<menuItems.size(); i++) {
      JMenuItem menuItem = (JMenuItem)menuItems.get(i);
      if(actionCommand.equals(menuItem.getActionCommand())) {
        ((JCheckBoxMenuItem)menuItem).setState(state);
        return;
      }
    }
    if(DEBUG)
      System.out.println("Men-Checkbox '"+actionCommand+"' nicht gefunden!");
  }

  /**
   * Liefert den Zustand einer Check-Box im Men.
   * @param actionCommand Schlssel zur Auswahl der Check-Box
   * @return aktueller Zustand der Check-Box
   */
  public boolean getState(String actionCommand) {
    for(int i=0; i<menuItems.size(); i++) {
      JMenuItem menuItem = (JMenuItem)menuItems.get(i);
      if(actionCommand.equals(menuItem.getActionCommand()))
        return ((JCheckBoxMenuItem)menuItem).getState();
    }
    if(DEBUG)
      System.out.println("Men-Checkbox '"+actionCommand+"' nicht gefunden!");
    return false; // eigentlich Exception!
  }

  /**
   * Setzt die Taste, mit der ein Menpunkt aufgerufen werden kann.
   * @param actionCommand Schlssel zur Auswahl des Menpunktes
   * @param code Tastencode (aus KeyEvent)
   * @see KeyEvent
   */
  public void setzeTaste(String actionCommand, int code) {
    for(int i=0; i<menuItems.size(); i++) {
      JMenuItem menuItem = (JMenuItem)menuItems.get(i);
      if(actionCommand.equals(menuItem.getActionCommand())) {
        menuItem.setMnemonic(code);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(code, ActionEvent.CTRL_MASK));
      }
    }
  }

  /**
   * Einzelne Menpunkte an- und ausschalten.
   * @param actionCommand Schlssel zur Auswahl des Menpunktes
   * @param b <code>true</code> Menpunkt an - <code>false</code> Menpunkt aus
   */
  public void setEnabled(String actionCommand, boolean b) {
    for(int i=0; i<menuItems.size(); i++) {
      JMenuItem menuItem = (JMenuItem)menuItems.get(i);
      if(actionCommand.equals(menuItem.getActionCommand()))
        menuItem.setEnabled(b);
    }
  }

  /**
   * Alle Menpunkte an- und ausschalten, die mit '#_' beginnen.
   * @param b <code>true</code> Menpunkte an - <code>false</code> Menpunkte aus
   */
  public void setSelectedEnabled(boolean b) {
    for(int i=0; i<menuItems.size(); i++) {
      JMenuItem menuItem = (JMenuItem)menuItems.get(i);
      if(menuItem.getActionCommand().startsWith("#_"))
        menuItem.setEnabled(b);
    }
  }

  /**
   * Fgt ein komplettes Men hinzu. Der erste Eintrag ist die Menberschrift,
   * gefolgt den einzelnen Menpunkten.<br>
   * <ul>
   * <li>"-" Separator
   * <li>"X_" Check-Box
   * <li>"#_" gesammelt deaktivierbar / aktivierbar
   * </ul>
   * @param text <code>[x][0]</code> actionCommand von Men Nr. x -
   *             <code>[x][1]</code> Mentext von Men Nr. x
   */
  public void addMenu(String[][] text) {
    JMenu menu = new JMenu(text[0][1]);
    menu.setActionCommand(text[0][0]);
    this.add(menu);
    menus.add(menu);
    for(int i=1; i<text.length; i++) {
      if(text[i][0].equals("-")) {
        menu.addSeparator();
      }
      else if(text[i][0].startsWith("X_")) {
        JCheckBoxMenuItem item = new JCheckBoxMenuItem(text[i][1]);
        item.setActionCommand(text[i][0]);
        menu.add(item);
        menuItems.add(item);
      }
      else
        addMenuItem(menu, text[i][1], text[i][0]);
    }
  }

  /**
   * Fgt einem bestehen Men einen weiteren Menpunkt hinzu.
   * <b>Aber keine Check-Box!</b>
   * @param actionCommandMenu Schlssel zur Auswahl des Mens
   * @param actionCommand     Schlssel des neuen Menpunktes
   * @param text              Menetext des neuen Menpunktes
   */
  public void extendMenu(String actionCommandMenu, String actionCommand, String text) {
    for(int i=0; i<menus.size(); i++) {
      JMenu menu = (JMenu)menus.get(i);
      if(DEBUG)
        System.out.println("Men Nr."+i+"="+menu.getActionCommand());
      if(actionCommandMenu.equals(menu.getActionCommand())) {
        addMenuItem(menu, text, actionCommand);
        return;
      }
    }
    Applikation.fehlerAusgabe.println("-E- extendMenu: "+actionCommandMenu+" nicht gefunden!");
  }

  private void addMenuItem(JMenu menu, String text, String actionCommand) {
    JMenuItem item = new JMenuItem(text);
    item.setActionCommand(actionCommand);
    item.addActionListener(this);
    menu.add(item);
    menuItems.add(item);
  }

  /**
   * Ruft die passende Funktion der Applikation bei Auswahl eines Menpunktes
   * auf. Check-Boxen werden ignoriert.
   * @see Applikation
   */

  public void actionPerformed(ActionEvent e) {
    String actionCommand = ((JMenuItem)(e.getSource())).getActionCommand();
    Class c = applikation.getClass();
    String menuBefehl = actionCommand;
    if(menuBefehl.startsWith("#_"))
      menuBefehl = actionCommand.substring(2);
    if(!menuBefehl.startsWith("X_")) {
      try {
        Method call = c.getMethod(menuBefehl, null);
        call.invoke(applikation, null);
      }
      catch (Exception e1) {
        Applikation.fehlerAusgabe.println("-E- bei: "+menuBefehl+" ("+e1.toString()+")");
        e1.printStackTrace();
      }
    }
  }

}