package app;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.print.*;
import java.util.StringTokenizer;
import java.util.ArrayList;

/**
 * Zeigt Tabellen an mit der Option sie auszudrucken. Die Trennung der
 * Spalten erfolgt per Tabulator '\\t'.<br>
 * <i>(C)opyright 2002 L.H.Hahn</i>
 * @author Dr. Lars H. Hahn
 * @version 3.1
 */

public class MultiPageViewer
  extends JDialog {
  private final static boolean DEBUG = false;

  private int aktuelleSeite = 1;
  private int letzteSeite;
  private int zeilenProSeite;
  private final ArrayList text = new ArrayList();
  private final ArrayList tabs = new ArrayList();
  private boolean originalTabs = true;
  private String ueberschrift;
  private final PageFormat seitenFormat;
  private Font font;
  private Font kursiv;
  private final PagePane pagePane;
  private final JPanel buttonPane = new JPanel();
  private final JButton next = new JButton("Nchste Seite");
  private final JButton prev = new JButton("Vorherige Seite");
  private final JButton drucken = new JButton("Drucken");
  private final JButton ende = new JButton("Beenden");

  /**
   * Einziger Konstruktor.
   * @param parent Eltern-Fenster
   * @param uberschrift Text, der im Dialog-Titel und in der Fuzeile des
   *        Dokumentes erscheint.
   * @param text Tabelle, die ausgegeben werden soll.
   * @param seitenFormat definiert das Seitenformat der Ausgabe
   */
  public MultiPageViewer(JFrame parent, String ueberschrift, PageFormat seitenFormat) {
    super(parent, ueberschrift, true);
    this.ueberschrift = ueberschrift;
    this.seitenFormat = seitenFormat;
    pagePane = new PagePane();
    setFont("SansSerif", 12);
    buttonPane.add(next);
    buttonPane.add(prev);
    buttonPane.add(drucken);
    buttonPane.add(ende);
    next.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        if(aktuelleSeite < letzteSeite) {
          aktuelleSeite++;
          pagePane.repaint();
        }
      }
    });
    prev.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        if(aktuelleSeite > 1) {
          aktuelleSeite--;
          pagePane.repaint();
        }
      }
    });
    drucken.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        drucken();
      }
    });
    ende.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        setVisible(false);
      }
    });
    JScrollPane scrollPane = new JScrollPane(pagePane);
    scrollPane.setPreferredSize(new Dimension((int)seitenFormat.getWidth(), 500));
    this.getContentPane().add(scrollPane, BorderLayout.CENTER);
    this.getContentPane().add(buttonPane, BorderLayout.SOUTH);
    this.clearText();
    this.clearTabs();
  }

  protected void buttonPaneAdd(JComponent comp) {
    buttonPane.add(comp);
  }
  /**
   * Zeigt den Dialog auf dem Bildschirm an, wenn er noch nicht sichtbar ist.
   */
  public void showDialog() {
    pack();
    setVisible(true);
  }

  /**
   * Verndert die berschrift, die im Dialog-Titel und in der Fuzeile des
   * Dokumentes erscheint.
   * @param ueberschrift neue berschrift
   */
  public void setUeberschrift(String ueberschrift) {
    this.ueberschrift = ueberschrift;
    this.setTitle(ueberschrift);
  }

  /**
   * Ermglicht im oberen Teil des Dialogs eine beliebige Pane anzuzeigen.
   * @param auswahlPane Pane zur Auswahl spezieller Parameter
   */
  public void setAuswahlPane(JPanel auswahlPane) {
    this.getContentPane().add(auswahlPane, BorderLayout.NORTH);
  }

  /**
   * ndert die Tabelle, die angezeigt wird. Spalten werden durch Tabulatoren
   * getrennt.
   * @param text Tabelle, die ausgegeben werden soll.
   */
  public void addText(String[] textblock) {
    text.add(textblock);
    seitenAnzahlBerechnen();
    pagePane.repaint();
  }

  /**
   * ndert die Tabelle, die angezeigt wird.
   * @param tab Tabelle, die ausgegeben werden soll.
   */
  public void addText(String[][] tab) {
    String[] textblock = new String[tab.length];
    for(int i=0; i<tab.length; i++) {
      textblock[i] = tab[i][0];
      for(int j=1; j<tab[i].length; j++) {
        textblock[i] += "\t"+tab[i][j];
      }
    }
    addText(textblock);
  }

  public String[][] getTextAt(int idx) {
    String[][] tab = null;
    String[] textblock = (String[])text.get(idx);
    int zeilen = textblock.length;
    int spalten = 0;
    for(int y=0; y<zeilen; y++) {
      StringTokenizer textzeile = new StringTokenizer(textblock[y], "\t");
      if(DEBUG)
        System.out.println("["+y+"] "+textblock[y]);
      if(y == 0) {
        spalten = textzeile.countTokens();
        tab = new String[zeilen][spalten];
      }
      for(int x=0; x<spalten; x++) {
        if(textzeile.hasMoreTokens())
          tab[y][x] = textzeile.nextToken();
        else
          tab[y][x] = "Ups! ["+y+"]["+x+"]";
      }
    }
    return tab;
  }

  public void clearText() {
    text.clear();
    aktuelleSeite = 1;
    letzteSeite = 1;
  }

  /**
   * Setzt die Tabulatoren der Tabelle.
   * @param relTabs relative Tabulatoren 0.0 bis 100.0
   */
  public void addTabs(double[] relTabs) {
    int anzahl = relTabs.length;
    int[] blocktabs = new int[anzahl];
    for(int i=0; i<anzahl; i++)
      blocktabs[i] = (int)(seitenFormat.getImageableWidth() * relTabs[i] / 100.0 + seitenFormat.getImageableX());
    if(originalTabs) {
      tabs.set(0, blocktabs);
      originalTabs = false;
    }
    else
      tabs.add(blocktabs);
  }

  public void clearTabs() {
    tabs.clear();
    double[] relTabs = {0.0, 10.0, 20.0, 30.0, 40.0, 50.0, 60.0, 70.0, 80.0, 90.0};
    originalTabs = false;
    this.addTabs(relTabs);
    originalTabs = true;
  }

  /**
   * Setzt den Zeichensatz der Tabelle.
   * @param name Name des Zeichensatzes
   * @param punkt Gre des Zeichensatzes (in Punkt)
   */
  public void setFont(String name, int punkt) {
    font = new Font(name, Font.PLAIN, punkt);
    kursiv = new Font(name, Font.ITALIC, punkt);
    seitenAnzahlBerechnen();
  }

  private void seitenAnzahlBerechnen() {
    int anzahlZeilen = text.size(); // Die Leer-Zeilen zwischen den Blcken
    for(int i=0; i<text.size(); i++) {
      String[] textblock = (String[])text.get(i);
      anzahlZeilen += textblock.length;
    }
    zeilenProSeite = (int)(seitenFormat.getImageableHeight() / pagePane.getFontMetrics(font).getHeight()) - 2;
    letzteSeite = anzahlZeilen / zeilenProSeite;
    if((anzahlZeilen - letzteSeite * zeilenProSeite) > 0)
      letzteSeite++;
    if(DEBUG)
      System.out.println("Zeilen="+anzahlZeilen+" Zeilen/Seite="+zeilenProSeite+" letzte Seite="+letzteSeite);
  }

  private void drucken() {
    PrinterJob job = PrinterJob.getPrinterJob();
    job.setJobName(ueberschrift);
    job.setPrintable(pagePane, seitenFormat);
    if(job.printDialog()) {
      try {
        job.print();
      }
      catch(PrinterException e) {
        Applikation.fehlerAusgabe.println("-E- Fehler beim Drucken!");
      }
    }
  }

  private String getAbsZeile(int seite, int zeile) {
    int gesuchteZeile = seite * zeilenProSeite + zeile;
    int aktuelleZeile = 0;
    for(int i=0; i<text.size(); i++) {
      String[] textblock = (String[])text.get(i);
      if((aktuelleZeile + textblock.length) > gesuchteZeile)
        return textblock[gesuchteZeile - aktuelleZeile];
      else if((aktuelleZeile + textblock.length) == gesuchteZeile)
        return ""; // Leerzeile nach dem Textblock
      aktuelleZeile += textblock.length + 1;
    }
    return null;
  }

  private int[] getLokaleTabs(int seite, int zeile) {
    int gesuchteZeile = seite * zeilenProSeite + zeile;
    int aktuelleZeile = 0;
    for(int i=0; i<text.size(); i++) {
      String[] textblock = (String[])text.get(i);
      if((aktuelleZeile + textblock.length + 1) > gesuchteZeile) {
        if(i < tabs.size())
          return (int[])tabs.get(i);
        else
          return (int[])tabs.get(tabs.size()-1);
      }
      aktuelleZeile += textblock.length + 1;
    }
    return (int[])tabs.get(tabs.size()-1);
  }

  private class PagePane
    extends JPanel
    implements Printable {

    public PagePane() {
      this.setPreferredSize(new Dimension((int)seitenFormat.getWidth(), (int)seitenFormat.getHeight()));
    }

    public void paintComponent(Graphics g) {
      setBackground(Color.white);
      super.paintComponent(g); //paint background
      print(g, seitenFormat, aktuelleSeite-1);
    }

    public int print(Graphics g, PageFormat pageFormat, int seite) {
      if(seite >= letzteSeite)
        return NO_SUCH_PAGE;
      g.setFont(font);
      int y = (int)(pageFormat.getImageableY() + g.getFontMetrics().getHeight() + 0.5D);
      for(int i=0; i<zeilenProSeite; i++) {
        String zeile = getAbsZeile(seite, i);
        int[] lokaleTabs = getLokaleTabs(seite, i);
        if(zeile != null) {
          int c = 0;
          StringTokenizer st = new StringTokenizer(zeile, "\t");
          while(st.hasMoreTokens()) {
            String wort = st.nextToken();
            int zellenBreite;
            if(c == (lokaleTabs.length-1))
              zellenBreite = (int)(seitenFormat.getImageableWidth() + seitenFormat.getImageableX()) - lokaleTabs[c];
            else
              zellenBreite = lokaleTabs[c+1] - lokaleTabs[c];
            while((g.getFontMetrics().stringWidth(wort) > zellenBreite) && (wort.length() > 1))
              wort = wort.substring(0, wort.length()-1);
            g.drawString(wort, lokaleTabs[c++], y);
            if(c == lokaleTabs.length)
              c--;
          }
        }
        y += g.getFontMetrics().getHeight();
      }
      // Fuzeile:
      g.setFont(kursiv);
      y += g.getFontMetrics().getHeight();
      int x = (int)(pageFormat.getImageableX() + 0.5D);
      g.drawString(ueberschrift, x, y);
      String seitenzahl = ""+(seite+1)+"/"+letzteSeite;
      x += (int)(pageFormat.getImageableWidth() - g.getFontMetrics().stringWidth(seitenzahl) + 0.5D);
      g.drawString(seitenzahl, x, y);
      return PAGE_EXISTS;
    }

  }

}